#ifndef __KPNC_ASPI32_H__
#define __KPNC_ASPI32_H__

//=[ ]====================================================
//
int (__cdecl *GetASPI32SupportInfoX) () = 0;
int (__cdecl *SendASPI32CommandX)(LPSRB* SRB) = 0;

//=[ ]========================================================
#define ASPI32_DLL_BASE_NAME	"WNASPI32.DLL"
#define ASPI32_DLL_ALT_NAME		"WNASPI32.DLL"

#define ASPI_DATA_IN	SRB_DIR_IN
#define ASPI_DATA_OUT	SRB_DIR_OUT
#define ASPI_TIME_OUT	900


#define ASPI_MAX_ID				8				// MAX   .
#define MAX_INFO_LEN			48
#define ASPI_SRB_LEN			0x100
#define ASPI_SENSE_LEN			SENSE_LEN		// in WINASPI32.h
#define ASPI_MAX_SENSE_SIZE		0x100
#define ASPI_SENSE_IN_POS		0x0

//=[e_ASPI32_NOT_FOUND]=======================================================
//
//		    ASPI32   
void e_ASPI32_NOT_FOUND(void) {
	fprintf(stderr,"-ERR:ASPI32 library not found\x7\n");
	_getch();
	exit(0);
}

//=[_ASPI32_INIT]=============================================================
//
//  ASPI32, -    ASPI32-DLL
//      , ..   -
//      -
//
int _ASPI32_INIT() {
	HINSTANCE h;
	if (SendASPI32CommandX) return 1;
	if ((h = LoadLibrary(ASPI32_DLL_BASE_NAME)) == 0) e_ASPI32_NOT_FOUND();
	SendASPI32CommandX =    (int (__cdecl *)(LPSRB*))GetProcAddress(h, "SendASPI32Command");
	GetASPI32SupportInfoX = (int (__cdecl *)())GetProcAddress(h, "GetASPI32SupportInfo");
	return 3;
}

//=[SEND_ASPI_CMD]============================================================
//
//     SCSI-     ASPI  
//    (   )
//
//	ARG:
//		adapter_id	-	  (0 - primary, 1 - secondary)
//		read_id		-	    (0 - master, 1 - slaeyer)
//		DB			-	CDB-,  
//		CDB_sz		-	 CDB-  
//		SENSE		-	   SENSE-
//		buf			-	  
//		buf_sz		-	    ( 64 )
//		flags		-	== ASPI_DATA_IN  :  --> 
//						== ASPI_DATA_OUT :   --> 
//
//	RET:
//						. SRB_Status (SS_COMP [0x1] if ALL OK)
//
//	NOTE:
//		-     9x/ME/NT/W2K/XP  __    
//		  . , ASPI-   
// 
#define _S_ERR_EVENT	"-ERR:CREATE EVENT\x7"
int SEND_ASPI_CMD(int adapter_id, int read_id, char *CDB, int CDB_sz, char *SENSE, 
									unsigned char *buf, int buf_sz, int flag)  {
	
	DWORD				ASPIStatus;
	DWORD				ASPIEventStatus;
	HANDLE				ASPICompletionEvent;
	SRB_ExecSCSICmd		*SRB;

	//  -     SCSI-
	if ((ASPICompletionEvent = CreateEvent(0,0,0,0)) == 0) return SS_ERR;

	//    SRB-
	SRB= (SRB_ExecSCSICmd*) malloc(sizeof(SRB_ExecSCSICmd));memset(SRB,0,sizeof(SRB_ExecSCSICmd));

	//  II
	if (SENSE) memset(SENSE, 'U', ASPI_MAX_SENSE_SIZE);

	//  SRB-
	SRB->SRB_Cmd		= SC_EXEC_SCSI_CMD;			//  SCSI 
	SRB->SRB_HaId		= (unsigned char)adapter_id;// ID 
	SRB->SRB_Flags		= flag | SRB_EVENT_NOTIFY;	//  
	SRB->SRB_Target		= read_id;					// ID 
	SRB->SRB_BufPointer	= buf;						//   
	SRB->SRB_BufLen		= buf_sz;					//  
	SRB->SRB_SenseLen	= ASPI_SENSE_LEN;			//  SENSE-
	SRB->SRB_CDBLen		= (unsigned char)CDB_sz;	//  ATAPI-
	SRB->SRB_PostProc	= ASPICompletionEvent;		//  
	memcpy(SRB->CDBByte, CDB, CDB_sz);				//  CDB-

	//  SCSI-  ASPI
	ASPIStatus			= SendASPI32CommandX ( (LPSRB*) SRB );

	//    ,   . 
	if (SRB->SRB_Status == SS_PENDING)
		ASPIEventStatus = WaitForSingleObject(ASPICompletionEvent,ASPI_TIME_OUT);
	
	//  -   
	if (ASPIEventStatus == WAIT_OBJECT_0) 
	{
		ResetEvent(ASPICompletionEvent);
		//CloseHandle(ASPICompletionEvent);
	}

	//  ,  SENSE-
	if (SENSE) memcpy(SENSE, SRB->SenseArea, SENSE_LEN);

	ASPIStatus = SRB->SRB_Status;
			CloseHandle(ASPICompletionEvent);
	
	free(SRB); 

	//   
	return ASPIStatus ;
}

#endif //__KPNC_ASPI32_H__
